import logging

import fastapi
import uvicorn
from image_service_foundation.logger import set_logging
from image_service_foundation.storage.models_provider import ModelsProvider
from image_detection_core.save_model import Model_Converter

from image_detection_service.api_v1.image_detection_service import (
    example_service_router as router_v1,
)
from image_detection_service.constants import (
    API_VERSION_PREFIX,
    DEFAULT_LOG_LEVEL,
    DEFAULT_SERVICE_PORT,
    DEFAULT_HOST,
    PROJECT_DESCRIPTION,
    PROJECT_NAME,
)
from image_detection_service.utils import (
    initialize_configuration,
    initialize_prometheus,
)
from image_detection_service.version import __version__

logger = logging.getLogger(__name__)

__all__ = ('run_app',)


def on_shutdown() -> None:
    logger.info('Killing app')
    logger.info('Goodbye!')


app = fastapi.FastAPI(title=PROJECT_NAME, version=__version__, description=PROJECT_DESCRIPTION)
app.include_router(router_v1, prefix=API_VERSION_PREFIX)
app.add_event_handler('shutdown', on_shutdown)


@app.get('/health')
async def health() -> dict:
    return {'status': 'OK'}


@app.get('/version')
async def version() -> dict:
    return {'version': __version__}


def run_app(port: int = None, log_level: str = None) -> None:
    config = initialize_configuration()

    models_provider = ModelsProvider(config)
    models_provider.download_models()

    port = port or config.get('exampletemplateservice.port', DEFAULT_SERVICE_PORT)
    host = config.get('exampletemplateservice.hostName', DEFAULT_HOST)
    log_file = config.get('exampletemplateservice.logFile')
    fastapi_log_level = config.get('fastapi.logLevel')
    log_level = log_level or config.get('exampletemplateservice.logLevel', DEFAULT_LOG_LEVEL)

    set_logging(filepath=log_file, level=log_level, multithreaded=True)
    logging.getLogger('multipart.multipart').setLevel(logging.WARNING)

    initialize_prometheus(config)

    logger.info(f'Starting app on {host}:{port}')
    uvicorn.run(app, host=host, port=port, log_level=fastapi_log_level)


if __name__ == '__main__':
    run_app(log_level='debug')



import prometheus_client as pc

__all__ = ('metrics',)


class ServiceMetrics:
    def __init__(self) -> None:
        self.calls_counter = pc.Counter(
            'example_template_service_calls', 'Amount of times the endpoint is called', labelnames=['endpoint']
        )

        self.failed_calls_counter = pc.Counter(
            'example_template_service_failed_calls',
            'Amount of times the endpoint is called & failed',
            labelnames=['endpoint'],
        )

    def called(self, endpoint: str) -> None:
        self.calls_counter.labels(endpoint).inc()

    def call_failed(self, endpoint: str) -> None:
        self.failed_calls_counter.labels(endpoint).inc()


metrics = ServiceMetrics()  # singleton; there should only be one



import logging
from typing import List

from fastapi import APIRouter, File, Request

from image_detection_service.metrics import metrics
from image_detection_service.service import DlImageAnalysisService
from image_detection_service.utils import initialize_configuration

import numpy as np
logger = logging.getLogger(__name__)


__all__ = ('example_service_router',)

service: DlImageAnalysisService


def on_startup() -> None:
    """Lazy loading of the service to avoid loading models too soon."""
    global service
    config = initialize_configuration()
    service = DlImageAnalysisService(config=config)


example_service_router = APIRouter()
example_service_router.add_event_handler('startup', on_startup)


@example_service_router.post('/example/sum', tags=['sum'])
async def addition(inputs: List[float]):

    logger.info(f'Received addition request (service)')
    metrics.called('/example/sum')
    try:
        response = service.add(data=inputs)
    except Exception:
        metrics.call_failed('/example/sum')
        raise

    logger.debug(f"Returning response {response}")

    return response

@example_service_router.post('/example/detect', tags=['detect'])
def detect(request: Request, claases_to_detect: str = None, frame: bytes = File(..., description='An arbitrary image.')) -> dict:
    try:
        metadata = request.query_params.__dict__['_dict']
    except Exception:
        metadata = {}

    try:
        image = np.fromstring(frame, np.uint8)
        response = service.detect(image, allowed_classes=claases_to_detect, metadata=metadata)
    except Exception as e:

        logger.error('Request failed.', str(e))
        raise

    logger.info(f'Returning response: {response}')
    return response



@example_service_router.get('/example/version', tags=['version'])
async def addition():

    logger.info(f'Received addition request (service)')
    metrics.called('/example/version')
    try:
        response = service.version()
    except Exception:
        metrics.call_failed('/example/version')
        raise

    logger.debug(f"Returning response {response}")

    return response






"""Start the Template Service Example from the command line."""
import argparse
import logging

import image_detection_service as service


logger = logging.getLogger(__name__)


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '-p',
        '--port',
        type=int,
        help='Specify the port to run the app on; evaluation order: '
        f'CLI argument -> YAML config -> code default ({service.DEFAULT_SERVICE_PORT})',
    )
    parser.add_argument(
        '-l',
        '--log-level',
        type=str,
        help='Log level to show',
        choices=['debug', 'info', 'warning', 'error', 'critical'],
    )
    parser.add_argument('-V', '--version', action='version', version=f'%(prog)s {service.__version__}')
    return parser.parse_args()


def run_cli() -> None:
    args = parse_args()
    service.run_app(port=args.port, log_level=args.log_level)


if __name__ == '__main__':
    service.run_app(port=service.DEFAULT_SERVICE_PORT)



import logging
import json
from typing import Optional
from image_detection_core.utils import check_number_input
from image_service_foundation.config.configuration_manager import (
    ConfigurationManager,
    ConfigurationMixin,
)
from image_detection_core.service.object_detection_service import ObjectDetectionService

from image_detection_service.constants import *
from image_detection_service.version import __version__

import cv2
import numpy as np

logger = logging.getLogger(__name__)

__all__ = ('DlImageAnalysisService',)


class DlImageAnalysisService(ConfigurationMixin):
    def __init__(self, config: ConfigurationManager = None) -> None:
        self.load_config(config=config, default_location=DEFAULT_CONFIG_PATH)
        self.input_size = config.get('yolo.input_size', 416)
        self.object_detection_service = ObjectDetectionService(config=config)


    def add(self, data) -> float:
        sum = 0
        print(data)
        print(f"We have {check_number_input(data)} numbers to add")
        for i in data:
            sum += i
            print(sum)

        print(f"Version of the service: {__version__}")
        return sum

    def version(self) -> float:
        print(f"Version of the service: {__version__}")
        v = __version__
        return v

    def detect(self, image: np.ndarray, allowed_classes: str = None, metadata: Optional[dict] = None) :

        img_np = cv2.imdecode(image, cv2.IMREAD_COLOR)  # cv2.IMREAD_COLOR in OpenCV 3.1
        original_image = cv2.cvtColor(img_np, cv2.COLOR_BGR2RGB)

        image_data = cv2.resize(original_image, (self.input_size, self.input_size))
        image_data = image_data / 255.
        images_data = []

        for i in range(1):
            images_data.append(image_data)

        images_data = np.asarray(images_data).astype(np.float32)
        detections = self.object_detection_service.detect(images_data, original_image,allowed_classes)
        return detections




